@@ -21,6 +21,7 @@
  * code by Owen Taylor <otaylor@redhat.com> and 
  * Carsten Haitzler <raster@rasterman.com>
  */
+#include <config.h>
 
 #include <math.h>
 #include <string.h>
@@ -87,6 +88,20 @@ match_theme_image (GtkStyle       *style,
   return NULL;
 }
 
+static GdkBitmap *
+get_window_for_shape (ThemeImage *image, GdkWindow *window, GtkWidget *widget)
+{
+  if (image->background_shaped)
+    {
+      if (GTK_IS_MENU (widget))
+	return gtk_widget_get_parent_window (widget);
+      else if (GTK_IS_WINDOW (widget))
+	return window;
+    }
+
+  return NULL;
+}
+
 static gboolean
 draw_simple_image(GtkStyle       *style,
 		  GdkWindow      *window,
@@ -129,35 +144,24 @@ draw_simple_image(GtkStyle       *style,
     {
       if (image->background)
 	{
-	  const gchar *name;
-	  GdkWindow *maskwin = NULL;
+	  GdkWindow *maskwin;
 	  GdkBitmap *mask = NULL;
+	  gboolean valid;
 
-          name = gtk_widget_get_name (widget);
-	  if (GTK_IS_MENU (widget))
-	    {
-	      maskwin = gtk_widget_get_parent_window (widget);
-	    }
-	  else if (GTK_IS_WINDOW (widget) &&
-		   (strcmp (name, "gtk-tooltips") == 0 ||
-		    strcmp (name, "hildon-navigator-list") == 0 ||
-		    strcmp (name, "hildon-status-bar-popup") == 0))
-	    {
-	      maskwin = window;
-	    }
-
+	  maskwin = get_window_for_shape (image, window, widget);
 	  if (maskwin)
 	    mask = gdk_pixmap_new (maskwin, width, height, 1);
 
-	  theme_pixbuf_render (image->background,
-			       window, mask, area,
-			       draw_center ? COMPONENT_ALL : COMPONENT_ALL | COMPONENT_CENTER,
-			       FALSE,
-			       x, y, width, height);
+	  valid = theme_pixbuf_render (image->background,
+				       window, mask, area,
+				       draw_center ? COMPONENT_ALL : COMPONENT_ALL | COMPONENT_CENTER,
+				       FALSE,
+				       x, y, width, height);
 
 	  if (mask)
 	    {
-	      gdk_window_shape_combine_mask (maskwin, mask, 0, 0);
+	      if (valid)
+		gdk_window_shape_combine_mask (maskwin, mask, 0, 0);
 	      g_object_unref (mask);
 	    }
 	}
@@ -889,7 +893,7 @@ draw_extension (GtkStyle       *style,
 static void
 draw_focus (GtkStyle     *style,
 	    GdkWindow    *window,
-	    GtkStateType  state_type,
+	    GtkStateType  state,
 	    GdkRectangle *area,
 	    GtkWidget    *widget,
 	    const gchar  *detail,
@@ -905,11 +909,12 @@ draw_focus (GtkStyle     *style,
 
   match_data.function = TOKEN_D_FOCUS;
   match_data.detail = (gchar *)detail;
-  match_data.flags = 0;
+  match_data.flags = THEME_MATCH_STATE;
+  match_data.state = state;
   
   if (!draw_simple_image (style, window, area, widget, &match_data, TRUE, FALSE,
 			  x, y, width, height))
-    parent_class->draw_focus (style, window, state_type, area, widget, detail,
+    parent_class->draw_focus (style, window, state, area, widget, detail,
 			      x, y, width, height);
 }
 
